import os

from wordcloud import WordCloud
from wordcloud import STOPWORDS

from app_store_review_etl.pipeline.steps.step import Step
from app_store_review_etl.settings import OUTPUTS_DIR
from app_store_review_etl.logger import logger


class WordCloudGraph(Step):
    def process(self, gspread_client, spreadsheet, inputs):
        logger.info('PLOTTING WORD CLOUD FROM REVIEWS...')

        worksheet = spreadsheet.worksheet('reviews')
        reviews = ''
        for review in worksheet.get('C2:C'):
            reviews += str(review[0])

        try:
            stopwords = set(STOPWORDS)
            stopwords.update(['app'])
            wc = WordCloud(
                background_color="white", max_words=2000, contour_width=3,
                contour_color='steelblue', stopwords=stopwords
            )

            wordcloud = wc.generate(reviews)
            image = wordcloud.to_image()
            output_filepath = os.path.join(OUTPUTS_DIR, 'wordcloud.png')
            image.save(output_filepath)

        except Exception as e:
            logger.debug(f'{type(e).__name__}: {e}')

        logger.info('COMPLETED PLOTTING WORD CLOUD.')



import sys
import getopt
sys.path.append('../')

from app_store_review_etl.google_sheets_auth.google_sheets_auth import GoogleSheetsAuth
from app_store_review_etl.pipeline.pipeline import Pipeline
from app_store_review_etl.pipeline.steps.preflight import Preflight
from app_store_review_etl.pipeline.steps.fetch_app_reviews import FetchAppReviews
from app_store_review_etl.pipeline.steps.sentiment_analysis import SentimentAnalysis
from app_store_review_etl.pipeline.steps.top_liked_disliked_analysis import TopLikedDislikedAnalysis
from app_store_review_etl.pipeline.steps.word_cloud_graph import WordCloudGraph
from app_store_review_etl.pipeline.steps.graph import Graph
from app_store_review_etl.pipeline.steps.update_graph import UpdateGraph
from app_store_review_etl.pipeline.steps.postflight import Postflight


APP_COUNTRY = 'us'
APP_NAME = 'youtube-watch-listen-stream'
DATE_AFTER = '2024-1-1'
AI_MODEL = 'gemini-1.0-pro-latest'


def print_usage():
    print('python3 main.py -c <app_country> -n <app_name> -d <date_after> -m <ai_model>')
    print('python3 main.py --app_country <app_country> --app_name <app_name> '
          '--date_after <date_after> --ai_model <ai_model>')

    print('python3 main.py OPTIONS')
    print('OPTIONS:')
    print('{:>10} {:<20}{}'.format('-c', '--app_country', 'The part of app store URL that indicates region or country'))
    print('{:>10} {:<20}{}'.format('-n', '--app_name', 'The part of app store URL that indicates the name of the app'))
    print('{:>10} {:<20}{}'.format('-d', '--date_after', 'Review recency. Default is fetching reviews after 2024-1-1'))
    print('{:>10} {:<20}{}'.format('-m', '--ai_model', 'Google AI model to be used. Default is gemini-1.0-pro-latest'))


def main():

    inputs = {
        'app_country': APP_COUNTRY,
        'app_name': APP_NAME,
        'date_after': DATE_AFTER,
        'ai_model': AI_MODEL,
    }

    short_opts = 'c:n:d:m:h'
    long_opts = 'help app_country= app_name= date_after= ai_model='.split()

    try:
        opts, args = getopt.getopt(sys.argv[1:], short_opts, long_opts)
    except getopt.GetoptError:
        print_usage()
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            print_usage()
            sys.exit(0)
        elif opt in ("-c", "--app_country"):
            inputs['app_country'] = arg
        elif opt in ("-n", "--app_name"):
            inputs['app_name'] = arg
        elif opt in ("-d", "--date_after"):
            inputs['date_after'] = arg
        elif opt in ("-m", "--ai_model"):
            inputs['ai_model'] = arg
        else:
            print_usage()
            sys.exit(2)

    steps = [
                Preflight,
                FetchAppReviews,
                SentimentAnalysis,
                TopLikedDislikedAnalysis,
                WordCloudGraph,
                Graph,
                UpdateGraph,
                Postflight,
            ]

    gspread_client = GoogleSheetsAuth.google_sheets_auth()
    spreadsheet_name = inputs['app_name'] + '_app_review_analysis'
    spreadsheet = gspread_client.create(spreadsheet_name)
    p = Pipeline(steps)
    p.run(gspread_client, spreadsheet, inputs)


if __name__ == "__main__":
    main()



import datetime
from app_store_scraper import AppStore

from app_store_review_etl.pipeline.steps.step import Step
from app_store_review_etl.logger import logger


class FetchAppReviews(Step):
    def process(self, gspread_client, spreadsheet, inputs):
        logger.info('FETCHING APP REVIEWS...')

        app_country = inputs['app_country']
        app_name = inputs['app_name']
        date_after = inputs['date_after']

        date_part = date_after.split('-')
        year = int(date_part[0])
        month = int(date_part[1])
        day = int(date_part[2])
        datatime = datetime.datetime(year, month, day)

        try:
            target_app = AppStore(country=app_country, app_name=app_name)
            target_app.review(after=datatime)

            header = ['Date', 'Rating', 'Review', 'UserName']
            data = [header]
            reviews = []

            for review in target_app.reviews:
                reviews.append(review)

            for review in reviews:
                data.append([str(review['date']), str(review['rating']), review['review'], review['userName']])

            title_worksheet = 'reviews'
            spreadsheet.add_worksheet(title=title_worksheet, rows=1000, cols=10)
            worksheet = spreadsheet.worksheet(title_worksheet)
            worksheet.update('A1', data)

        except Exception as e:
            logger.info(f'{type(e).__name__}: {e}')

        logger.info('COMPLETED FETCHING APP REVIEWS.')



import gspread

from app_store_review_etl.settings import CREDENTIALS_GSPREAD_FILE_PATH


class GoogleSheetsAuth:
    @staticmethod
    def google_sheets_auth():
        scopes = ['https://www.googleapis.com/auth/drive.file']

        gc = gspread.oauth(
            credentials_filename=CREDENTIALS_GSPREAD_FILE_PATH,
            authorized_user_filename='../authorized_user.json',
            scopes=scopes,
        )

        return gc



import os
import re

from google.oauth2.credentials import Credentials
from googleapiclient.discovery import build
from googleapiclient.http import MediaFileUpload

from app_store_review_etl.pipeline.steps.step import Step
from app_store_review_etl.logger import logger


class UpdateGraph(Step):
    def process(self, gspread_client, spreadsheet, inputs):
        logger.info('UPDATING GRAPHS INTO GOOGLE SHEET...')

        credentials = Credentials.from_authorized_user_file('../authorized_user.json')
        drive_service = build('drive', 'v3', credentials=credentials)

        title_worksheet = 'data_graphs'
        spreadsheet.add_worksheet(title=title_worksheet, rows=1000, cols=10)
        worksheet3 = spreadsheet.worksheet(title_worksheet)

        for file_name in os.listdir('./outputs'):
            file_metadata = {'name': file_name}
            media = MediaFileUpload(f'./outputs/{file_name}', mimetype='image/png')
            try:
                uploaded_image = drive_service.files().create(body=file_metadata, media_body=media,
                                                              fields='id').execute()
                permission = {
                    'type': 'anyone',
                    'role': 'reader'
                }
                drive_service.permissions().create(fileId=uploaded_image['id'], body=permission).execute()
                image_url = f'https://drive.google.com/uc?export=view&id={uploaded_image["id"]}'
                logger.info(f'uploaded image URL: {image_url}')

                res = worksheet3.append_row([file_name, '=IMAGE(\"{}\")'.format(image_url)],
                                            value_input_option='USER_ENTERED')
                width = 480
                height = 360
                row = int(re.findall("\\w+![A-Z]+([0-9]+)", res['updates']['updatedRange'], re.S)[0])
                requests = [{
                    "updateDimensionProperties": {
                        "properties": {
                            "pixelSize": height
                        },
                        "range": {
                            "sheetId": worksheet3.id,
                            "dimension": "ROWS",
                            "startIndex": row - 1,
                            "endIndex": row
                        },
                        "fields": "pixelSize"
                    }
                },
                    {
                        "updateDimensionProperties": {
                            "properties": {
                                "pixelSize": width
                            },
                            "range": {
                                "sheetId": worksheet3.id,
                                "dimension": "COLUMNS",
                                "startIndex": 1,
                                "endIndex": 2
                            },
                            "fields": "pixelSize"
                        }
                    }
                ]
                spreadsheet.batch_update({"requests": requests})

            except Exception as e:
                logger.debug(f'{type(e).__name__}: {e}')

        logger.info('COMPLETED UPDATING GRAPHS INTO GOOGLE SHEET.')



from app_store_review_etl.pipeline.steps.step import Step
from app_store_review_etl.logger import logger


class Postflight(Step):
    def process(self, gspread_client, spreadsheet, inputs):
        logger.info('IN POSTFLIGHT...')
        sheet1 = spreadsheet.worksheet('Sheet1')
        spreadsheet.del_worksheet_by_id(sheet1.id)
        logger.info('ALL COMPLETED!')
        logger.info(f'Output Google Sheet is ready: {spreadsheet.url}')



from vaderSentiment.vaderSentiment import SentimentIntensityAnalyzer

from app_store_review_etl.pipeline.steps.step import Step
from app_store_review_etl.logger import logger


class SentimentAnalysis(Step):
    def process(self, gspread_client, spreadsheet, inputs):
        logger.info('ANALYZING SENTIMENT...')

        worksheet = spreadsheet.worksheet('reviews')

        try:
            header = ['Polarity Score', 'Sentiment', 'Sentiment Emoji']
            data = [header]

            analyzer = SentimentIntensityAnalyzer()
            for review in worksheet.get('C2:C'):
                score_dict = analyzer.polarity_scores(review[0])
                compound_score = score_dict['compound']
                sentiment = self.classify_sentiment(compound_score)
                sentiment_emoji = self.classify_sentiment_emoji(compound_score)
                data.append([str(compound_score), str(sentiment), str(sentiment_emoji)])

            worksheet.update('E:G', data)
            worksheet.freeze(rows=1)
            worksheet.sort((1, 'des'))

        except Exception as e:
            logger.debug(f'{type(e).__name__}: {e}')

        logger.info('COMPLETED ANALYZING SENTIMENT.')

    def classify_sentiment(self, compound_score):
        if compound_score >= 0.05:
            return 'Positive'
        elif compound_score <= -0.05:
            return 'Negative'
        else:
            return 'Neutral'

    def classify_sentiment_emoji(self, compound_score):
        if compound_score >= 0.05:
            return '\U0001f600'
        elif compound_score <= -0.05:
            return '\U0001F621'
        else:
            return '\U0001F610'



import os

import matplotlib.pyplot as plt

from app_store_review_etl.pipeline.steps.step import Step
from app_store_review_etl.settings import OUTPUTS_DIR
from app_store_review_etl.logger import logger


class Graph(Step):
    def process(self, gspread_client, spreadsheet, inputs):
        logger.info('PLOTTING DISTRIBUTION GRAPHS...')

        # read data from Sheet1 and store data in a dictionary
        worksheet = spreadsheet.worksheet('reviews')
        try:
            ratings = []
            for rating in worksheet.get('B2:B'):
                ratings.append(str(rating[0]))

            polarity_scores = []
            for polarity in worksheet.get('E2:E'):
                polarity_scores.append(str(polarity[0]))

            sentiments = []
            for sentiment in worksheet.get('F2:F'):
                sentiments.append(str(sentiment[0]))

            data = {
                'ratings': ratings,
                'polarity_scores': polarity_scores,
                'sentiments': sentiments
            }

            # plot distribution of sentiment
            sentiments_data = {
                'Negative': data['sentiments'].count('Negative'),
                'Neutral': data['sentiments'].count('Neutral'),
                'Positive': data['sentiments'].count('Positive')
            }
            fig, ax = plt.subplots()
            sentiment_type = list(sentiments_data.keys())
            sentiment_count = list(sentiments_data.values())
            ax.bar(sentiment_type, sentiment_count, color=['tomato', 'gold', 'forestgreen'])
            ax.set_title('Distribution of sentiment')
            ax.set_xlabel('Sentiment')
            ax.set_ylabel('Count')
            output_filepath = os.path.join(OUTPUTS_DIR, 'sentiment.png')
            plt.savefig(output_filepath, dpi=500)

            # plot distribution of polarity score
            polarity_scores = [float(score) for score in data['polarity_scores']]
            fig, ax = plt.subplots()
            ax.hist(polarity_scores, bins=10, color='slategrey')
            ax.set_title('Distribution of polarity score')
            ax.set_xlabel('Polarity')
            ax.set_ylabel('Count')
            output_filepath = os.path.join(OUTPUTS_DIR, 'polarity.png')
            plt.savefig(output_filepath, dpi=500)

            # plot distribution of app rating
            rating_data = {
                '5': data['ratings'].count('5'),
                '4': data['ratings'].count('4'),
                '3': data['ratings'].count('3'),
                '2': data['ratings'].count('2'),
                '1': data['ratings'].count('1')
            }
            fig, ax = plt.subplots()
            rating = list(rating_data.keys())
            count = list(rating_data.values())
            ax.barh(rating, count, color='lightsteelblue')
            ax.set_title('Distribution of app rating')
            ax.set_xlabel('Count')
            ax.set_ylabel('Rating')
            output_filepath = os.path.join(OUTPUTS_DIR, 'rating.png')
            plt.savefig(output_filepath, dpi=500)

        except Exception as e:
            logger.debug(f'{type(e).__name__}: {e}')

        logger.info('COMPLETED PLOTTING DISTRIBUTION GRAPHS.')



from app_store_review_etl.pipeline.steps.step import Step
from app_store_review_etl.utils import Utils
from app_store_review_etl.logger import logger


class Preflight(Step):
    def process(self, gspread_client, spreadsheet, inputs):
        logger.info('IN PREFLIGHT...')
        utils = Utils()
        utils.create_dirs()



import google.generativeai as genai
from nltk.tokenize.casual import TweetTokenizer

from app_store_review_etl.pipeline.steps.step import Step
from app_store_review_etl.settings import GEMINI_API_KEY
from app_store_review_etl.logger import logger


class TopLikedDislikedAnalysis(Step):
    def process(self, gspread_client, spreadsheet, inputs):
        logger.info('ANALYZING TOP LIKES/DISLIKES FEATURES BY GEMINI MODEL...')

        # google gemini ai model config
        genai.configure(api_key=GEMINI_API_KEY)
        ai_model = inputs['ai_model']
        model = genai.GenerativeModel(ai_model)

        # read reviews from Sheet1 and generate prompt
        try:
            worksheet = spreadsheet.worksheet('reviews')
            final_reviews = []
            sum_tokens = 0
            count_reviews = 0
            for review in worksheet.get('C2:C'):
                review_tokens = self.count_token(review[0])
                sum_tokens += (review_tokens + 2)
                if sum_tokens < 28000:
                    count_reviews += 1
                    logger.info(f'sum_tokens: {sum_tokens}, count_reviews: {count_reviews}')
                    final_reviews.append(review[0])
                else:
                    break

            prompt = 'Given the following app reviews list, please generate ' \
                     'top ten things people like and top ten things people dislike ' \
                     'about the app into two numbered lists in English, listed from the most ' \
                     'prominent to the least prominent:\n' + str(final_reviews)

            logger.info(f'prompt: {prompt}')
            logger.info(f'{model.count_tokens(prompt)}')

            # get response and update response to google sheet
            response = model.generate_content(
                contents=[prompt]
            )
            output_text = response.text
            logger.info(f'{output_text}')
            output_data = []
            output_text_list = output_text.strip().split('\n')
            for output_text in output_text_list:
                output_data.append([output_text])

            title_worksheet = 'top_analysis'
            spreadsheet.add_worksheet(title=title_worksheet, rows=1000, cols=10)
            worksheet2 = spreadsheet.worksheet(title_worksheet)
            worksheet2.update('A1', output_data)

        except Exception as e:
            logger.info(f'{type(e).__name__}: {e}')

        logger.info('COMPLETED ANALYZING TOP LIKES/DISLIKES FEATURES.')

    def count_token(self, review):
        tokenizer = TweetTokenizer()
        token_list = tokenizer.tokenize(review)
        token_count = len(token_list)
        return token_count



import os
from dotenv import load_dotenv

load_dotenv()

CREDENTIALS_GSPREAD_FILE_PATH = os.getenv('CREDENTIALS_GSPREAD_FILE_PATH')
GEMINI_API_KEY = os.getenv('GEMINI_API_KEY')
OUTPUTS_DIR = 'outputs'



from app_store_review_etl.pipeline.steps.step import StepException


class Pipeline:
    def __init__(self, steps):
        self.steps = steps

    def run(self, gspread_client, spreadsheet, inputs):
        for step in self.steps:
            step_instance = step()
            try:
                step_instance.process(gspread_client, spreadsheet, inputs)
            except StepException as e:
                print(e)
                break
